@@ -1,3 +1,4 @@
+import math
 from functools import partial, reduce
 from typing import Union
 
@@ -33,6 +34,11 @@ class UnsignedBVExpr:
         def _indexer(idx):
             return cmn.index_gate(self.size, idx, self.output, cmn._fresh())
 
+        if isinstance(idx, int):
+            if abs(idx) >= self.size:
+                raise ValueError(
+                    'Index %d out of range (BV size %d).' % (idx, self.size))
+            idx = idx % self.size
         if not isinstance(idx, slice):
             idx = slice(idx, idx+1)
         if idx.start is None:
@@ -154,6 +160,33 @@ class UnsignedBVExpr:
     def _fresh_output(self):
         return self.with_output(cmn._fresh())
 
+    def __mul__(self, other):
+        if isinstance(other, int):
+            signed = isinstance(self, SignedBVExpr)
+
+            size = math.ceil(math.log2(abs(other) + 1))
+            size += int(signed)  # Signed has double the range.
+
+            other = atom(size, other, signed=signed)
+
+        if isinstance(other, SignedBVExpr) != isinstance(self, SignedBVExpr):
+            raise ValueError('Cannot multiply signed with unsigned integers.')
+
+        # Determine the smaller circuit to minimize worst-case depth.
+        smaller, larger = sorted([self, other], key=lambda x: x.size)
+        result = atom(larger.size, 0, signed=isinstance(self, SignedBVExpr))
+        for i in range(smaller.size):
+            mask = smaller[i].repeat(larger.size)
+            delta = mask & (larger << i)
+
+            # For signed multiplication, need to subtract the last index.
+            subtract = i == smaller.size - 1 and isinstance(self, SignedBVExpr)
+            if subtract:
+                result -= delta
+            else:
+                result += delta
+        return result
+
 
 class SignedBVExpr(UnsignedBVExpr):
     def __neg__(self):
@@ -171,6 +204,9 @@ class SignedBVExpr(UnsignedBVExpr):
     def __abs__(self):
         return _unary_gate(cmn.abs_gate, self)
 
+    def sign(self):
+        return UnsignedBVExpr(self.aigbv)[-1]
+
 
 Expr = Union[UnsignedBVExpr, SignedBVExpr]
 
